# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Copyright (c) <2013> <Colin Duquesnoy>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
"""
Contains QPropertyGrid implementation.
"""
from PyQt4 import QtGui, QtCore
from qpropertygrid.widgets import QColorButton

BUILTIN_TYPES = [bool, int, float, str, QtGui.QColor, QtGui.QFont]


class QPropertyGrid(QtGui.QTableWidget):
    """
    A simple property grid capable of displaying and editing the values
    of a regular python dictionary.
    """

    #: Signal emitted when a builtin widget is added to the grid. This let
    #: you customize the added widget.
    widgetAdded = QtCore.pyqtSignal(QtGui.QWidget)

    #: Signal emitted when a value changed. Parameters are the key, the
    # old value and the new value
    valueChanged = QtCore.pyqtSignal(str, object, object)

    def __init__(self, parent=None, custom_data_callback=None):
        """
        :param custom_data_callback: Callback function called when a unknown
            data type is found in the dict.
        """
        super().__init__(parent)
        hv = self.horizontalHeader()
        hv.setStretchLastSection(True)
        hv.setResizeMode(QtGui.QHeaderView.ResizeToContents)
        self.verticalHeader().hide()
        self.setHorizontalHeader(hv)
        self.setAlternatingRowColors(True)
        self._data = {}
        self._custom_data_callback = custom_data_callback

    @property
    def data(self):
        """
        Gets/Sets the data. Data is a regular python dictionary.
        """
        return self._data

    @data.setter
    def data(self, d):
        assert isinstance(d, dict)
        self._data = d
        self._refresh_grid()

    def set_value(self, key, value):
        """
        Sets the value of key in the data dict.

        :param key: The key of the value to change
        :param value: The new value.
        """
        old_value = self._data[key]
        if value != old_value:
            self._data[key] = value
            self.valueChanged.emit(key, self._data[key], value)

    def _refresh_grid(self):
        self.clear()
        self.setColumnCount(2)
        self.setRowCount(0)
        self.setHorizontalHeaderLabels(["Property", "Value"])
        for key in sorted(self.data.keys()):
            row = self.rowCount()
            # create label
            value = self._data[key]
            label = QtGui.QLabel(key)
            label.setContentsMargins(3, 0, 0, 0)

            # create widget
            widget = None
            if type(value) in BUILTIN_TYPES:
                widget = self._make_builtin_widget(value)
            elif self._custom_data_callback is not None:
                widget = self._custom_data_callback(value)
            # insert widgets in a new row
            if widget:
                widget.key = key
                try:
                    widget.fr
                except AttributeError:
                    pass
                self.widgetAdded.emit(widget)
                self.insertRow(row)
                self.setCellWidget(row, 0, label)
                self.setCellWidget(row, 1, widget)
        self.horizontalHeader().adjustSize()

    def _make_builtin_widget(self, value):
        widget = None
        if isinstance(value, QtGui.QFont):
            widget = QtGui.QFontComboBox()
            widget.setCurrentFont(QtGui.QFont(value))
            widget.currentFontChanged.connect(self._on_font_changed)
        elif isinstance(value, bool):
            widget = QtGui.QCheckBox()
            widget.setChecked(value)
            widget.stateChanged.connect(self._on_bool_changed)
        elif isinstance(value, float):
            widget = QtGui.QDoubleSpinBox()
            widget.setValue(value)
            widget.editingFinished.connect(self._on_float_changed)
        elif isinstance(value, int):
            widget = QtGui.QSpinBox()
            widget.setValue(value)
            widget.editingFinished.connect(self._on_int_changed)
        elif isinstance(value, QtGui.QColor):
            widget = QColorButton()
            widget.color = value
            widget.valueChanged.connect(self._on_color_changed)
            return widget
        elif isinstance(value, str):
            widget = QtGui.QLineEdit()
            widget.setText(value)
            widget.editingFinished.connect(self._on_str_changed)
        return widget

    def _on_font_changed(self):
        widget = self.sender()
        key = widget.key
        value = widget.currentFont()
        self.set_value(key, value)

    def _on_bool_changed(self, state):
        key = self.sender().key
        self.set_value(key, state)

    def _on_int_changed(self):
        key = self.sender().key
        self.set_value(key, self.sender().value())

    def _on_float_changed(self):
        key = self.sender().key
        self.set_value(key, self.sender().value())

    def _on_str_changed(self):
        key = self.sender().key
        value = self.sender().text()
        self.set_value(key, value)

    def _on_color_changed(self, value):
        key = self.sender().key
        self.set_value(key, value)

    def setBackgroundColor(self, color):
        darker = color.darker(110)
        self.setStyleSheet("QPropertyGrid{"
                           "    alternate-background-color: %s;"
                           "    background-color: %s;"
                           "}" % (color.name(), darker.name()))
